@@ -675,10 +675,19 @@ db_src_probe (GstPad * pad, GstPadProbeInfo * info, OutputPad * output)
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_EOS:
     {
+      gboolean drop_event = FALSE;
+      GstPlayItem *last;
+
+      PLAY_ITEMS_LOCK (uridecodebin);
+      last = g_list_last (uridecodebin->play_items)->data;
+
       /* If there is a next input, drop the EOS event */
       if (uridecodebin->input_item != uridecodebin->output_item ||
-          uridecodebin->input_item !=
-          g_list_last (uridecodebin->play_items)->data) {
+          uridecodebin->input_item != last) {
+        drop_event = TRUE;
+      }
+      PLAY_ITEMS_UNLOCK (uridecodebin);
+      if (drop_event) {
         GST_DEBUG_OBJECT (uridecodebin,
             "Dropping EOS event because in gapless mode");
         return GST_PAD_PROBE_DROP;
@@ -1600,9 +1609,7 @@ gst_uri_decode_bin3_set_property (GObject * object, guint prop_id,
 
   switch (prop_id) {
     case PROP_URI:
-      PLAY_ITEMS_LOCK (dec);
       gst_uri_decode_bin3_set_uri (dec, g_value_get_string (value));
-      PLAY_ITEMS_UNLOCK (dec);
       break;
     case PROP_SUBURI:
       gst_uri_decode_bin3_set_suburi (dec, g_value_get_string (value));
@@ -1934,7 +1941,6 @@ uri_src_ignore_block_probe (GstPad * pad, GstPadProbeInfo * info,
   return GST_PAD_PROBE_OK;
 }
 
-/* PLAY_ITEMS_LOCK held */
 static void
 gst_uri_decode_bin3_set_uri (GstURIDecodeBin3 * dec, const gchar * uri)
 {
@@ -1943,6 +1949,7 @@ gst_uri_decode_bin3_set_uri (GstURIDecodeBin3 * dec, const gchar * uri)
 
   GST_DEBUG_OBJECT (dec, "uri: %s", uri);
 
+  PLAY_ITEMS_LOCK (dec);
   item = next_inactive_play_item (dec);
   play_item_set_uri (item, uri);
 
@@ -1985,6 +1992,7 @@ gst_uri_decode_bin3_set_uri (GstURIDecodeBin3 * dec, const gchar * uri)
     }
   }
 
+  PLAY_ITEMS_UNLOCK (dec);
   if (start_item) {
     /* Start new item */
     activate_play_item (item);
@@ -2185,18 +2193,21 @@ gst_uri_decode_bin3_handle_redirection (GstURIDecodeBin3 * uridecodebin,
     goto beach;
 
   if (g_strcmp0 (current_uri, uri)) {
-    gboolean was_instant = uridecodebin->instant_uri;
+    gst_message_unref (message);
+    message = NULL;
+
     /* We only want to handle the redirection once */
     if (!handler->saw_redirection) {
+      gboolean was_instant = uridecodebin->instant_uri;
       handler->saw_redirection = TRUE;
       GST_DEBUG_OBJECT (uridecodebin, "Doing instant switch to '%s'", uri);
       uridecodebin->instant_uri = TRUE;
       /* Force instant switch */
+      PLAY_ITEMS_UNLOCK (uridecodebin);
       gst_uri_decode_bin3_set_uri (uridecodebin, uri);
+      PLAY_ITEMS_LOCK (uridecodebin);
       uridecodebin->instant_uri = was_instant;
     }
-    gst_message_unref (message);
-    message = NULL;
   }
   g_free (uri);
 
@@ -2211,6 +2222,35 @@ gst_uri_decode_bin3_handle_message (GstBin * bin, GstMessage * msg)
   GstURIDecodeBin3 *uridecodebin = (GstURIDecodeBin3 *) bin;
 
   switch (GST_MESSAGE_TYPE (msg)) {
+    case GST_MESSAGE_STREAM_COLLECTION:
+    {
+      GstSourceHandler *handler;
+      GST_DEBUG_OBJECT (uridecodebin, "Handle stream collection");
+      PLAY_ITEMS_LOCK (uridecodebin);
+      /* Find the matching handler (if any) */
+      if ((handler = find_source_handler_for_element (uridecodebin, msg->src))) {
+        gboolean selectable = FALSE;
+        GstQuery *query = gst_query_new_selectable ();
+        /* We only want to forward this message if the source is selectable,
+         * else we will let decodebin3 do its emission. */
+        if (gst_element_query ((GstElement *) msg->src, query)) {
+          gst_query_parse_selectable (query, &selectable);
+          GST_DEBUG_OBJECT (uridecodebin,
+              "%s is selectable : %d",
+              GST_ELEMENT_NAME (handler->urisourcebin), selectable);
+        }
+        gst_query_unref (query);
+        if (!selectable) {
+          GST_DEBUG_OBJECT (uridecodebin,
+              "%s doesn't handle selection, dropping stream-collection message",
+              GST_ELEMENT_NAME (handler->urisourcebin));
+          gst_message_unref (msg);
+          msg = NULL;
+        }
+      }
+      PLAY_ITEMS_UNLOCK (uridecodebin);
+      break;
+    }
     case GST_MESSAGE_STREAMS_SELECTED:
     {
       GstSourceHandler *handler;
